@@ -1,5 +1,5 @@
 # this is hack to make sure no escape characters are inserted into the output
-import os;
+import os
 if 'TERM' in os.environ:
     del os.environ['TERM']
 import doctest
@@ -15,5 +15,8 @@ doctest.testmod(mercurial.httprepo)
 import mercurial.util
 doctest.testmod(mercurial.util)
 
+import mercurial.dagparser
+doctest.testmod(mercurial.dagparser, optionflags=doctest.NORMALIZE_WHITESPACE)
+
 import hgext.convert.cvsps
 doctest.testmod(hgext.convert.cvsps)
